#!/bin/bash

##########################################################################################
# This script gathers container image metadata from ONAP container images and Nordix
# onap_docker_images.list file and identifies if any image image changed or not by
# comparing image sha256sums. If it identifies changes in any of the images, it then
# tags the new version of the image with Nordix tag, pushes the image to Nordix Harbor
# and adds image metadata to new onap_docker_images.list file.
# As a final step, onap_docker_images.list file gets uploaded to Nordix Artifactory.
#
# Checks done by the script
#
# if image exists in nordix
#   if not, tags image with ONAP_TAG.0, pushes it and adds entry to filelist
#   if so, checks SHA of the ONAP container image to see if it is same with Nordix container image
#     if not, checks if the ONAP_TAG matches to the first 3 digits of NORDIX_TAG
#       if not, tags the image with ONAP_TAG.0, pushes it and adds entry to filelist
#       if so, tags image with ONAP_TAG.{NORDIX_DIGIT+1}, pushes it and adds entry to filelist
#     if so, checks if the ONAP_TAG matches to the first 3 digits of NORDIX_TAG
#       if not, tags image with ONAP_TAG.0, pushes it and adds entry to filelist
#       if so, adds entry to filelist without pushing and tagging
##########################################################################################

main() {
  # declare arrays to keep image metadata
  declare -A ONAP_IMAGE ONAP_IMAGES
  declare -A NORDIX_IMAGE NORDIX_IMAGES

  # ensure we don't have leftovers from previous execution
  /bin/rm -f /tmp/onap_docker_images.list.temp && touch /tmp/onap_docker_images.list.temp

  # gather ONAP container image metadata from docker
  while read -r line; do
    IMAGE_NAME=$(echo $line | cut -d'#' -f1)
    ONAP_IMAGE['sha']=$(echo $line | cut -d'#' -f3)
    ONAP_IMAGE['tag']=$(echo $line | cut -d'#' -f2)
    ONAP_IMAGE_METADATA=$(declare -p ONAP_IMAGE)
    ONAP_IMAGES[$IMAGE_NAME]=${ONAP_IMAGE_METADATA}
  done < <(docker images --no-trunc --format "{{.Repository}}#{{.Tag}}#{{.ID}}" | grep nexus | sort)

  # download Nordix ONAP filelist
  /bin/rm -f $WORKSPACE/onap_docker_images.list
  curl -s "$NORDIX_ONAP_ARTIFACTORY_REPOSITORY/latest/onap_docker_images.list" \
      -o $WORKSPACE/onap_docker_images.list

  # gather Nordix container image metadata from onap_docker_images.list file
  while read line; do
    IMAGE_NAME=$(echo $line | cut -d':' -f1)
    NORDIX_IMAGE['sha']=$(echo $line | cut -d'#' -f2)
    NORDIX_IMAGE['tag']=$(echo $line | cut -d':' -f2 | sed 's|\s.*||g')
    NORDIX_IMAGE_METADATA=$(declare -p NORDIX_IMAGE)
    NORDIX_IMAGES[$IMAGE_NAME]=${NORDIX_IMAGE_METADATA}
  done < $WORKSPACE/onap_docker_images.list

  # compare ONAP and Nordix container images
  for ONAP_IMAGE_NAME in ${!ONAP_IMAGES[@]}; do

    # get ONAP image metadata
    eval ${ONAP_IMAGES[${ONAP_IMAGE_NAME}]}
    ONAP_IMAGE_TAG=${ONAP_IMAGE['tag']}
    ONAP_IMAGE_SHA=${ONAP_IMAGE['sha']}

    # get Nordix image metadata
    NORDIX_IMAGE_NAME=$(echo $ONAP_IMAGE_NAME | sed "s|^nexus[^/]*|$NORDIX_ONAP_HARBOR_REPOSITORY|g")
    eval ${NORDIX_IMAGES[${NORDIX_IMAGE_NAME}]}

    # check if image exists on Nordix
    if [ "${NORDIX_IMAGES[$NORDIX_IMAGE_NAME]+abc}" ]; then
      NORDIX_IMAGE_SHA=${NORDIX_IMAGE['sha']}
      NORDIX_IMAGE_CURRENT_TAG=${NORDIX_IMAGE['tag']}
      NORDIX_IMAGE_CURRRENT_TAG_PREFIX=${NORDIX_IMAGE_CURRENT_TAG%.*}
      NORDIX_IMAGE_CURRRENT_TAG_AFFIX=$(echo $NORDIX_IMAGE_CURRENT_TAG | cut -d'.' -f4)

      # check if ONAP and Nordix Image SHAs match
      if [ "$ONAP_IMAGE_SHA" != "$NORDIX_IMAGE_SHA" ]; then
        if [ "$NORDIX_IMAGE_CURRRENT_TAG_PREFIX" != "$ONAP_IMAGE_TAG" ]; then
          NORDIX_IMAGE_NEW_TAG="$ONAP_IMAGE_TAG.0"
        else
          NORDIX_IMAGE_NEW_TAG="$ONAP_IMAGE_TAG.$((NORDIX_IMAGE_CURRRENT_TAG_AFFIX+1))"
        fi
        tag_push_image $ONAP_IMAGE_NAME $ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME $NORDIX_IMAGE_NEW_TAG
      else
        # check if there is a new tag
        if [ "$NORDIX_IMAGE_CURRRENT_TAG_PREFIX" != "$ONAP_IMAGE_TAG" ]; then
          echo "New tag!"
          NORDIX_IMAGE_NEW_TAG="$ONAP_IMAGE_TAG.0"
          tag_push_image $ONAP_IMAGE_NAME $ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME $NORDIX_IMAGE_NEW_TAG
        else
          NORDIX_IMAGE_NEW_TAG="$NORDIX_IMAGE_CURRENT_TAG"
        fi
      fi
    else
      # image first time appearing in Nordix
      NORDIX_IMAGE_NEW_TAG="$ONAP_IMAGE_TAG.0"
      tag_push_image $ONAP_IMAGE_NAME $ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME $NORDIX_IMAGE_NEW_TAG
    fi
    echo "${NORDIX_IMAGE_NAME}:${NORDIX_IMAGE_NEW_TAG} #${ONAP_IMAGE_SHA}" >> /tmp/onap_docker_images.list.temp
  done

  store_onap_filelist /tmp/onap_docker_images.list.temp
}

tag_push_image() {
  ONAP_IMAGE_NAME=$1
  ONAP_IMAGE_TAG=$2
  NORDIX_IMAGE_NAME=$3
  NORDIX_IMAGE_TAG=$4

  # tag and push using Nordix tag
  docker tag $ONAP_IMAGE_NAME:$ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME:$NORDIX_IMAGE_TAG
  docker push $NORDIX_IMAGE_NAME:$NORDIX_IMAGE_TAG

  # tag and push using ONAP tag
  docker tag $ONAP_IMAGE_NAME:$ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME:$ONAP_IMAGE_TAG
  docker push $NORDIX_IMAGE_NAME:$ONAP_IMAGE_TAG

  # tag and push using ONAP tag
  docker tag $ONAP_IMAGE_NAME:$ONAP_IMAGE_TAG $NORDIX_IMAGE_NAME:$ONAP_VERSION
  docker push $NORDIX_IMAGE_NAME:$ONAP_VERSION

  echo "Tagged and pushed"
  echo "    $NORDIX_IMAGE_NAME:$NORDIX_IMAGE_TAG"
  echo "    $NORDIX_IMAGE_NAME:$ONAP_IMAGE_TAG"
  echo "    $NORDIX_IMAGE_NAME:$ONAP_VERSION"
}

store_onap_filelist() {
  ONAP_FILELIST=$1

  # calculate md5sum, shasum, sha256sum
  ONAP_FILELIST_MD5SUM=$(md5sum $ONAP_FILELIST | cut -d' ' -f1)
  ONAP_FILELIST_SHASUM=$(shasum $ONAP_FILELIST | cut -d' ' -f1)
  ONAP_FILELIST_SHA256SUM=$(sha256sum $ONAP_FILELIST | cut -d' ' -f1)

  # store the file under date/timestamp and latest folders
  PROMOTION_ID=$(date +%Y%m%d%H%M%S)
  for subfolder in $PROMOTION_ID latest; do
    curl -s \
        -H "X-JFrog-Art-Api:$NORDIX_ARM_TOKEN" \
        -H "X-Checksum-MD5:$ONAP_FILELIST_MD5SUM" \
        -H "X-Checksum-Sha1:$ONAP_FILELIST_SHASUM" \
        -H "X-Checksum-Sha256:$ONAP_FILELIST_SHA256SUM" \
        -T $ONAP_FILELIST \
        "$NORDIX_ONAP_ARTIFACTORY_REPOSITORY/${subfolder}/onap_docker_images.list"
  done
}

main "$@"
